; One-time sensor network project
; Power measurment module firmware
; (C) 2007 Thomas Bleeker
; www.madwizard.org
;
; For ATtiny26
;
.include <tn26def.inc>

.org 0x0000
	rjmp start
.org OVF1addr
	rjmp ovf1

.equ PIN_TXD = PB1

.cseg

.def rTemp 		= r16
.def rMode 		= r17
.def rBitNr 	= r18
.def rData 		= r19
.def rSeqNr 	= r5
.def rNextBits 	= r20

; Timer 1 overflow interrupt
ovf1:
	; Hold other interrupts in handler
	cli

	; Output the next UART bit
	out  PORTB, rNextBits

	; Set default bit to send the next
	; time to 1.
	ldi  rNextBits, (1<<PIN_TXD)

	; Bit number 9? Then a stop bit is needed
	cpi  rBitNr, 9
	breq stop_bit

	; Bit zero? Load a new set of bits
	tst  rBitNr
	brne data_bit

	; Start with next bit is zero
	clr  rNextBits
	tst  rMode
	brne mode_data
		; Sequence number mode, load current
		; sequence number into rData
		mov  rData, rSeqNr
		inc  rSeqNr
		rjmp done
	mode_data:
		; Data mode, start the analog-digital converter
		in   rData, ADCH
		sbi  ADCSR, ADSC ; Start new conversion
		rjmp done

data_bit:
	; Shift out the next databit and 
	; place it in rNextBits
	lsr 	rData
	brcs 	bitset
	clr 	rNextBits
	bitset:
	rjmp 	done
	
	
stop_bit:
	; Stop bit, switch mode and make sure
	; bitNr is zero next time.
	eor rMode, r1
	ser rBitNr ; becomes 0 after increment
done:
	; next bit
	inc rBitNr
	
	; Return from interrupt
	reti	

; --- main program ---
start:

	; Setup global vars
	clr rBitNr
	clr rMode
	clr rData

	; Initialize analog-digital converter
	ldi  rTemp, (1<<REFS1)|(1<<ADLAR)
	out  ADMUX, rTemp
	ori  rTemp, 0b00001011 
	out  ADMUX, rTemp
	ldi  rTemp, (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)
	out  ADCSR, rTemp

	; Initialize next bits
	ldi  rNextBits, (1<<PIN_TXD)
	clr  rTemp
	
	; Initialize stack
	ldi  rTemp, RAMEND
	out  SP, rTemp	

	; Start with sequence number zero
	clr  rSeqNr

	; Initialize output pin
	out  PORTB, rNextBits
	out  DDRB, rNextBits
	
	; Setup timer 1
	ldi  rTemp, (1<<CTC1)|(1<<CS10)  ;no prescaler, reset on compare match
	out  TCCR1B, rTemp

	; Set timer interval in cycles (to match UART speed)
	ldi  rTemp, 69 - 1
	out  OCR1C, rTemp
	
	; Enable timer interrupt
	ldi	 rTemp, (1<<TOIE1)
	out  TIMSK, rTemp

	; r0 = 0, r1 = 1
	clr  r0
	clr  r1
	inc  r1

	; Enable interrupts
	sei

	; Loop forever. The rest will be handled
	; in the interrupt handler
	stuck: rjmp stuck

